@@ -19,9 +19,9 @@
 
 #include "config.h"
 
-#include "gdk-private.h"
+#include "gdkprivate.h"
 #include "gdkdragsurfaceprivate.h"
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
 
 /**
  * GdkDragSurface:
