@@ -520,8 +520,6 @@ _gdk_wayland_display_create_surface (GdkDisplay     *display,
   surface->width = width;
   surface->height = height;
 
-  g_object_ref (surface);
-
   /* More likely to be right than just assuming 1 */
   if (display_wayland->compositor_version >= WL_SURFACE_HAS_BUFFER_SCALE)
     {
@@ -632,7 +630,7 @@ gdk_wayland_surface_dispose (GObject *object)
         GDK_WAYLAND_DISPLAY (gdk_surface_get_display (surface));
 
       display_wayland->event_queues =
-        g_list_remove (display_wayland->event_queues, surface);
+        g_list_remove (display_wayland->event_queues, impl->event_queue);
       g_clear_pointer (&impl->event_queue, wl_event_queue_destroy);
     }
 
