@@ -36,22 +36,15 @@
 #include "gtkbutton.h"
 #include "gtkgrid.h"
 #include "gtkbox.h"
-#include "gtkicontheme.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
 #include "gtkstack.h"
-#include "gtkorientable.h"
-#include "gtkscrolledwindow.h"
 #include "gtktextview.h"
-#include "gtkmain.h"
-#include "gtktogglebutton.h"
 #include "gtktypebuiltins.h"
 #include "gtkstack.h"
-#include "gtkstackswitcher.h"
 #include "gtksettings.h"
 #include "gtkurilauncher.h"
-#include "gtkheaderbar.h"
 #include "gtkprivate.h"
 #include <glib/gi18n-lib.h>
 #include "gtkeventcontrollermotion.h"
