@@ -393,8 +393,9 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
 static void
 update_button_sensitivity (GtkFontDialogButton *self)
 {
-  gtk_widget_set_sensitive (self->button,
-                            self->dialog != NULL && self->cancellable == NULL);
+  if (self->button)
+    gtk_widget_set_sensitive (self->button,
+                              self->dialog != NULL && self->cancellable == NULL);
 }
 
 static void
@@ -402,10 +403,11 @@ family_chosen (GObject      *source,
                GAsyncResult *result,
                gpointer      data)
 {
+  GtkFontDialog *dialog = GTK_FONT_DIALOG (source);
   GtkFontDialogButton *self = data;
   PangoFontFamily *family;
 
-  family = gtk_font_dialog_choose_family_finish (self->dialog, result, NULL);
+  family = gtk_font_dialog_choose_family_finish (dialog, result, NULL);
   if (family)
     {
       PangoFontDescription *desc;
@@ -428,10 +430,11 @@ face_chosen (GObject      *source,
              GAsyncResult *result,
              gpointer      data)
 {
+  GtkFontDialog *dialog = GTK_FONT_DIALOG (source);
   GtkFontDialogButton *self = data;
   PangoFontFace *face;
 
-  face = gtk_font_dialog_choose_face_finish (self->dialog, result, NULL);
+  face = gtk_font_dialog_choose_face_finish (dialog, result, NULL);
   if (face)
     {
       PangoFontDescription *desc;
@@ -453,10 +456,11 @@ font_chosen (GObject      *source,
              GAsyncResult *result,
              gpointer      data)
 {
+  GtkFontDialog *dialog = GTK_FONT_DIALOG (source);
   GtkFontDialogButton *self = data;
   PangoFontDescription *desc;
 
-  desc = gtk_font_dialog_choose_font_finish (self->dialog, result, NULL);
+  desc = gtk_font_dialog_choose_font_finish (dialog, result, NULL);
   if (desc)
     {
       gtk_font_dialog_button_set_font_desc (self, desc);
@@ -472,12 +476,13 @@ font_and_features_chosen (GObject      *source,
                           GAsyncResult *result,
                           gpointer      data)
 {
+  GtkFontDialog *dialog = GTK_FONT_DIALOG (source);
   GtkFontDialogButton *self = data;
   PangoFontDescription *desc;
   char *features;
   PangoLanguage *language;
 
-  if (gtk_font_dialog_choose_font_and_features_finish (self->dialog, result,
+  if (gtk_font_dialog_choose_font_and_features_finish (dialog, result,
                                                        &desc, &features, &language,
                                                        NULL))
     {
@@ -628,10 +633,9 @@ update_font_info (GtkFontDialogButton *self)
 
       gtk_label_set_text (GTK_LABEL (self->size_label), size);
       g_free (size);
-      gtk_widget_show (self->font_size_box);
     }
-  else
-    gtk_widget_hide (self->font_size_box);
+
+  gtk_widget_set_visible (self->font_size_box, self->level >= GTK_FONT_LEVEL_FONT);
 }
 
 static void
@@ -697,7 +701,7 @@ gtk_font_dialog_button_new (GtkFontDialog *dialog)
 {
   GtkWidget *self;
 
-  g_return_val_if_fail (GTK_IS_FONT_DIALOG (dialog), NULL);
+  g_return_val_if_fail (dialog == NULL || GTK_IS_FONT_DIALOG (dialog), NULL);
 
   self = g_object_new (GTK_TYPE_FONT_DIALOG_BUTTON,
                        "dialog", dialog,
@@ -838,7 +842,7 @@ gtk_font_dialog_button_set_font_desc (GtkFontDialogButton        *self,
  * Returns the font of the button.
  *
  * This function is what should be used to obtain
- * the font that was choosen by the user. To get
+ * the font that was chosen by the user. To get
  * informed about changes, listen to "notify::font-desc".
  *
  * Returns: (transfer none) (nullable): the font
@@ -889,7 +893,7 @@ gtk_font_dialog_button_set_font_features (GtkFontDialogButton *self,
  * Returns the font features of the button.
  *
  * This function is what should be used to obtain the font features
- * that were choosen by the user. To get informed about changes, listen
+ * that were chosen by the user. To get informed about changes, listen
  * to "notify::font-features".
  *
  * Note that the button will only let users choose font features
