@@ -69,7 +69,7 @@
  *
  * Within a widget, gestures can be grouped through [method@Gtk.Gesture.group].
  * Grouped gestures synchronize the state of sequences, so calling
- * [method@Gtk.Gesture.set_sequence_state] on one will effectively propagate
+ * [method@Gtk.Gesture.set_state] on one will effectively propagate
  * the state throughout the group.
  *
  * By default, all sequences start out in the %GTK_EVENT_SEQUENCE_NONE state,
@@ -99,7 +99,7 @@
  * again.
  *
  * Sequence states can't be changed freely.
- * See [method@Gtk.Gesture.set_sequence_state] to know about the possible
+ * See [method@Gtk.Gesture.set_state] to know about the possible
  * lifetimes of a `GdkEventSequence`.
  *
  * ## Touchpad gestures
@@ -120,7 +120,6 @@
 #include "gtktypebuiltins.h"
 #include "gtkprivate.h"
 #include "gtkmain.h"
-#include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtknative.h"
 
@@ -162,7 +161,6 @@ struct _GtkGesturePrivate
   GdkDevice *device;
   GList *group_link;
   guint n_points;
-  guint hold_timeout_id;
   guint recognized : 1;
   guint touchpad : 1;
 };
@@ -175,8 +173,6 @@ static guint signals[N_SIGNALS] = { 0 };
                                       gdk_event_get_event_type (e) == GDK_TOUCHPAD_PINCH || \
                                       gdk_event_get_event_type (e) == GDK_TOUCHPAD_HOLD)
 
-#define HOLD_TIMEOUT_MS 50
-
 GList * _gtk_gesture_get_group_link (GtkGesture *gesture);
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GtkGesture, gtk_gesture, GTK_TYPE_EVENT_CONTROLLER)
@@ -225,7 +221,6 @@ gtk_gesture_finalize (GObject *object)
 
   gtk_gesture_ungroup (gesture);
   g_list_free (priv->group_link);
-  g_clear_handle_id (&priv->hold_timeout_id, g_source_remove);
 
   g_hash_table_destroy (priv->points);
 
@@ -524,7 +519,9 @@ _gtk_gesture_update_point (GtkGesture     *gesture,
       else
         state = gtk_gesture_get_group_state (gesture, sequence);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_gesture_set_sequence_state (gesture, sequence, state);
+G_GNUC_END_IGNORE_DEPRECATIONS
     }
 
   return TRUE;
@@ -583,22 +580,6 @@ _gtk_gesture_cancel_all (GtkGesture *gesture)
   _gtk_gesture_check_empty (gesture);
 }
 
-static gboolean
-gtk_gesture_hold_timeout (gpointer user_data)
-{
-  GtkGesture *gesture;
-  GtkGesturePrivate *priv;
-
-  gesture = user_data;
-  priv = gtk_gesture_get_instance_private (gesture);
-
-  if (priv->touchpad)
-    _gtk_gesture_cancel_sequence (gesture, priv->last_sequence);
-
-  priv->hold_timeout_id = 0;
-  return G_SOURCE_REMOVE;
-}
-
 static gboolean
 gesture_within_surface (GtkGesture *gesture,
                         GdkSurface  *surface)
@@ -660,14 +641,10 @@ gtk_gesture_handle_event (GtkEventController *controller,
 
   if (event_type == GDK_BUTTON_PRESS ||
       event_type == GDK_TOUCH_BEGIN ||
-      (event_type == GDK_TOUCHPAD_SWIPE && phase == GDK_TOUCHPAD_GESTURE_PHASE_BEGIN) ||
-      (event_type == GDK_TOUCHPAD_PINCH && phase == GDK_TOUCHPAD_GESTURE_PHASE_BEGIN) ||
-      (event_type == GDK_TOUCHPAD_HOLD && phase == GDK_TOUCHPAD_GESTURE_PHASE_BEGIN))
+      (EVENT_IS_TOUCHPAD_GESTURE (event) &&
+       phase == GDK_TOUCHPAD_GESTURE_PHASE_BEGIN &&
+       gdk_touchpad_event_get_n_fingers (event) == priv->n_points))
     {
-      if ((event_type == GDK_TOUCHPAD_PINCH || event_type == GDK_TOUCHPAD_SWIPE) &&
-          _gtk_gesture_has_matching_touchpoints (gesture))
-        g_clear_handle_id (&priv->hold_timeout_id, g_source_remove);
-
       if (_gtk_gesture_update_point (gesture, event, target, x, y, TRUE))
         {
           gboolean triggered_recognition;
@@ -697,9 +674,9 @@ gtk_gesture_handle_event (GtkEventController *controller,
     }
   else if (event_type == GDK_BUTTON_RELEASE ||
            event_type == GDK_TOUCH_END ||
-           (event_type == GDK_TOUCHPAD_SWIPE && phase == GDK_TOUCHPAD_GESTURE_PHASE_END) ||
-           (event_type == GDK_TOUCHPAD_PINCH && phase == GDK_TOUCHPAD_GESTURE_PHASE_END) ||
-           (event_type == GDK_TOUCHPAD_HOLD && phase == GDK_TOUCHPAD_GESTURE_PHASE_END))
+           (EVENT_IS_TOUCHPAD_GESTURE (event) &&
+            phase == GDK_TOUCHPAD_GESTURE_PHASE_END &&
+            gdk_touchpad_event_get_n_fingers (event) == priv->n_points))
     {
       gboolean was_claimed = FALSE;
 
@@ -719,8 +696,9 @@ gtk_gesture_handle_event (GtkEventController *controller,
     }
   else if (event_type == GDK_MOTION_NOTIFY ||
            event_type == GDK_TOUCH_UPDATE ||
-           (event_type == GDK_TOUCHPAD_SWIPE && phase == GDK_TOUCHPAD_GESTURE_PHASE_UPDATE) ||
-           (event_type == GDK_TOUCHPAD_PINCH && phase == GDK_TOUCHPAD_GESTURE_PHASE_UPDATE))
+           (EVENT_IS_TOUCHPAD_GESTURE (event) &&
+            phase == GDK_TOUCHPAD_GESTURE_PHASE_UPDATE &&
+            gdk_touchpad_event_get_n_fingers (event) == priv->n_points))
     {
       if (event_type == GDK_MOTION_NOTIFY)
         {
@@ -737,21 +715,13 @@ gtk_gesture_handle_event (GtkEventController *controller,
       if (!priv->touchpad)
         _gtk_gesture_cancel_sequence (gesture, sequence);
     }
-  else if ((event_type == GDK_TOUCHPAD_SWIPE && phase == GDK_TOUCHPAD_GESTURE_PHASE_CANCEL) ||
-           (event_type == GDK_TOUCHPAD_PINCH && phase == GDK_TOUCHPAD_GESTURE_PHASE_CANCEL))
+  else if (EVENT_IS_TOUCHPAD_GESTURE (event) &&
+           phase == GDK_TOUCHPAD_GESTURE_PHASE_CANCEL &&
+           gdk_touchpad_event_get_n_fingers (event) == priv->n_points)
     {
       if (priv->touchpad)
         _gtk_gesture_cancel_sequence (gesture, sequence);
     }
-  else if (event_type == GDK_TOUCHPAD_HOLD && phase == GDK_TOUCHPAD_GESTURE_PHASE_CANCEL)
-    {
-      if (priv->hold_timeout_id == 0)
-        {
-          priv->hold_timeout_id = g_timeout_add (HOLD_TIMEOUT_MS,
-                                                 gtk_gesture_hold_timeout,
-                                                 gesture);
-        }
-    }
   else if (event_type == GDK_GRAB_BROKEN)
     {
       GdkSurface *surface;
@@ -777,10 +747,6 @@ gtk_gesture_handle_event (GtkEventController *controller,
 static void
 gtk_gesture_reset (GtkEventController *controller)
 {
-  GtkGesture *gesture = GTK_GESTURE (controller);
-  GtkGesturePrivate *priv = gtk_gesture_get_instance_private (gesture);
-
-  g_clear_handle_id (&priv->hold_timeout_id, g_source_remove);
   _gtk_gesture_cancel_all (GTK_GESTURE (controller));
 }
 
@@ -949,7 +915,6 @@ gtk_gesture_init (GtkGesture *gesture)
   priv->points = g_hash_table_new_full (NULL, NULL, NULL,
                                         (GDestroyNotify) free_point_data);
   priv->group_link = g_list_prepend (NULL, gesture);
-  priv->hold_timeout_id = 0;
 }
 
 /**
@@ -1054,6 +1019,8 @@ gtk_gesture_get_sequence_state (GtkGesture       *gesture,
  *
  * Returns: %TRUE if @sequence is handled by @gesture,
  *   and the state is changed successfully
+ *
+ * Deprecated: 4.10. Use [method@Gtk.Gesture.set_state]
  */
 gboolean
 gtk_gesture_set_sequence_state (GtkGesture            *gesture,
@@ -1106,8 +1073,46 @@ gtk_gesture_set_sequence_state (GtkGesture            *gesture,
  * Sets the state of all sequences that @gesture is currently
  * interacting with.
  *
- * See [method@Gtk.Gesture.set_sequence_state] for more details
- * on sequence states.
+ * Sequences start in state %GTK_EVENT_SEQUENCE_NONE, and whenever
+ * they change state, they can never go back to that state. Likewise,
+ * sequences in state %GTK_EVENT_SEQUENCE_DENIED cannot turn back to
+ * a not denied state. With these rules, the lifetime of an event
+ * sequence is constrained to the next four:
+ *
+ * * None
+ * * None → Denied
+ * * None → Claimed
+ * * None → Claimed → Denied
+ *
+ * Note: Due to event handling ordering, it may be unsafe to set the
+ * state on another gesture within a [signal@Gtk.Gesture::begin] signal
+ * handler, as the callback might be executed before the other gesture
+ * knows about the sequence. A safe way to perform this could be:
+ *
+ * ```c
+ * static void
+ * first_gesture_begin_cb (GtkGesture       *first_gesture,
+ *                         GdkEventSequence *sequence,
+ *                         gpointer          user_data)
+ * {
+ *   gtk_gesture_set_state (first_gesture, GTK_EVENT_SEQUENCE_CLAIMED);
+ *   gtk_gesture_set_state (second_gesture, GTK_EVENT_SEQUENCE_DENIED);
+ * }
+ *
+ * static void
+ * second_gesture_begin_cb (GtkGesture       *second_gesture,
+ *                          GdkEventSequence *sequence,
+ *                          gpointer          user_data)
+ * {
+ *   if (gtk_gesture_get_sequence_state (first_gesture, sequence) == GTK_EVENT_SEQUENCE_CLAIMED)
+ *     gtk_gesture_set_state (second_gesture, GTK_EVENT_SEQUENCE_DENIED);
+ * }
+ * ```
+ *
+ * If both gestures are in the same group, just set the state on
+ * the gesture emitting the event, the sequence will be already
+ * be initialized to the group's global state when the second
+ * gesture processes the event.
  *
  * Returns: %TRUE if the state of at least one sequence
  *   was changed successfully
@@ -1127,8 +1132,10 @@ gtk_gesture_set_state (GtkGesture            *gesture,
   priv = gtk_gesture_get_instance_private (gesture);
   sequences = g_hash_table_get_keys (priv->points);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   for (l = sequences; l; l = l->next)
     handled |= gtk_gesture_set_sequence_state (gesture, l->data, state);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   g_list_free (sequences);
 
