@@ -22,7 +22,8 @@
 #include "gtkmediafileprivate.h"
 
 #include "gtkdebug.h"
-#include "gtkintl.h"
+#include "gtkprivate.h"
+#include <glib/gi18n-lib.h>
 #include "gtkmodulesprivate.h"
 #include "gtknomediafileprivate.h"
 
@@ -72,7 +73,7 @@ gtk_media_file_default_open (GtkMediaFile *self)
 static void
 gtk_media_file_default_close (GtkMediaFile *self)
 {
-  gtk_media_stream_unprepared (GTK_MEDIA_STREAM (self));
+  gtk_media_stream_stream_unprepared (GTK_MEDIA_STREAM (self));
 }
 
 static void
@@ -155,9 +156,7 @@ gtk_media_file_class_init (GtkMediaFileClass *class)
    * The file being played back or %NULL if not playing a file.
    */
   properties[PROP_FILE] =
-    g_param_spec_object ("file",
-                         P_("File"),
-                         P_("File being played back"),
+    g_param_spec_object ("file", NULL, NULL,
                          G_TYPE_FILE,
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
@@ -169,9 +168,7 @@ gtk_media_file_class_init (GtkMediaFileClass *class)
    * This is %NULL when playing a file.
    */
   properties[PROP_INPUT_STREAM] =
-    g_param_spec_object ("input-stream",
-                         P_("Input stream"),
-                         P_("Input stream being played back"),
+    g_param_spec_object ("input-stream", NULL, NULL,
                          G_TYPE_INPUT_STREAM,
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
@@ -190,7 +187,7 @@ gtk_media_file_get_extension (void)
   GIOExtension *e;
   GIOExtensionPoint *ep;
 
-  GTK_NOTE (MODULES, g_print ("Looking up MediaFile extension\n"));
+  GTK_DEBUG (MODULES, "Looking up MediaFile extension");
 
   ep = g_io_extension_point_lookup (GTK_MEDIA_FILE_EXTENSION_POINT_NAME);
   e = NULL;
@@ -250,7 +247,8 @@ gtk_media_file_get_impl_type (void)
   e = gtk_media_file_get_extension ();
   impl_type = g_io_extension_get_type (e);
 
-  GTK_NOTE (MODULES, g_print ("Using %s from \"%s\" extension\n", g_type_name (impl_type), g_io_extension_get_name (e)));
+  GTK_DEBUG (MODULES, "Using %s from \"%s\" extension",
+             g_type_name (impl_type), g_io_extension_get_name (e));
 
   return impl_type;
 }
@@ -270,7 +268,7 @@ gtk_media_file_new (void)
 
 /**
  * gtk_media_file_new_for_filename:
- * @filename: filename to open
+ * @filename: (type filename): filename to open
  *
  * Creates a new media file for the given filename.
  *
@@ -420,7 +418,7 @@ gtk_media_file_clear (GtkMediaFile *self)
 /**
  * gtk_media_file_set_filename:
  * @self: a `GtkMediaFile`
- * @filename: (nullable): name of file to play
+ * @filename: (type filename) (nullable): name of file to play
  *
  * Sets the `GtkMediaFile to play the given file.
  *
@@ -613,8 +611,7 @@ gtk_media_file_extension_init (void)
   char **paths;
   int i;
 
-  GTK_NOTE (MODULES,
-            g_print ("Registering extension point %s\n", GTK_MEDIA_FILE_EXTENSION_POINT_NAME));
+  GTK_DEBUG (MODULES, "Registering extension point %s", GTK_MEDIA_FILE_EXTENSION_POINT_NAME);
 
   ep = g_io_extension_point_register (GTK_MEDIA_FILE_EXTENSION_POINT_NAME);
   g_io_extension_point_set_required_type (ep, GTK_TYPE_MEDIA_FILE);
@@ -626,8 +623,7 @@ gtk_media_file_extension_init (void)
   paths = _gtk_get_module_path ("media");
   for (i = 0; paths[i]; i++)
     {
-      GTK_NOTE (MODULES,
-                g_print ("Scanning io modules in %s\n", paths[i]));
+      GTK_DEBUG (MODULES, "Scanning io modules in %s", paths[i]);
       g_io_modules_scan_all_in_directory_with_scope (paths[i], scope);
     }
   g_strfreev (paths);
