@@ -19,7 +19,7 @@ main (int argc, char *argv[])
   grid = gtk_grid_new ();
   gtk_grid_set_row_spacing (GTK_GRID (grid), 30);
   gtk_grid_set_column_spacing (GTK_GRID (grid), 30);
-  gtk_container_add (GTK_CONTAINER (window), grid);
+  gtk_window_set_child (GTK_WINDOW (window), grid);
 
   label1 = gtk_label_new ("Some Text");
   label2 = gtk_label_new ("QQQQQQQQQ");
@@ -52,7 +52,7 @@ main (int argc, char *argv[])
   gtk_grid_attach (GTK_GRID (grid), label3, 0, 1, 1, 1);
   gtk_grid_attach (GTK_GRID (grid), label4, 1, 1, 1, 1);
 
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
   while (TRUE)
     g_main_context_iteration (NULL, TRUE);
   return 0;
