@@ -5,6 +5,7 @@
 #define COLOR " red;"
 
 static const char *css =
+" window { background-color: white; }\n"
 ".one {"
 "  all: unset;"
 "  min-width: 100px;"
@@ -164,7 +165,7 @@ main (int argc, char **argv)
   gtk_box_append (GTK_BOX (box), bottom);
   gtk_window_set_child (GTK_WINDOW (window), box);
   g_signal_connect (window, "destroy", G_CALLBACK (quit_cb), &done);
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
 
   while (!done)
     g_main_context_iteration (NULL, TRUE);
