@@ -284,8 +284,8 @@ static const char *buffer =
 "              <object class='GtkLabel' id='left'>"
 "                <property name='label'>Left</property>"
 "                <layout>"
-"                  <property name='left-attach'>0</property>"
-"                  <property name='top-attach'>0</property>"
+"                  <property name='column'>0</property>"
+"                  <property name='row'>0</property>"
 "                </layout>"
 "              </object>"
 "            </child>"
@@ -293,8 +293,8 @@ static const char *buffer =
 "              <object class='GtkLabel' id='right'>"
 "                <property name='label'>Right</property>"
 "                <layout>"
-"                  <property name='left-attach'>2</property>"
-"                  <property name='top-attach'>0</property>"
+"                  <property name='column'>2</property>"
+"                  <property name='row'>0</property>"
 "                </layout>"
 "              </object>"
 "            </child>"
@@ -305,8 +305,8 @@ static const char *buffer =
 "                 <property name='hexpand'>True</property>"
 "                 <property name='vexpand'>True</property>"
 "                <layout>"
-"                  <property name='left-attach'>1</property>"
-"                  <property name='top-attach'>0</property>"
+"                  <property name='column'>1</property>"
+"                  <property name='row'>0</property>"
 "                </layout>"
 "              </object>"
 "            </child>"
@@ -538,28 +538,28 @@ main (int argc, char *argv[])
                                               GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
   win1 = test_nonzerox ();
-  gtk_widget_show (win1);
+  gtk_window_present (GTK_WINDOW (win1));
 
   win2 = test_relative ();
-  gtk_widget_show (win2);
+  gtk_window_present (GTK_WINDOW (win2));
 
   win3 = test_fullwidth ();
-  gtk_widget_show (win3);
+  gtk_window_present (GTK_WINDOW (win3));
 
   win4 = test_scrolling ();
-  gtk_widget_show (win4);
+  gtk_window_present (GTK_WINDOW (win4));
 
   win5 = test_builder ();
-  gtk_widget_show (win5);
+  gtk_window_present (GTK_WINDOW (win5));
 
   win6 = test_chase ();
-  gtk_widget_show (win6);
+  gtk_window_present (GTK_WINDOW (win6));
 
   win7 = test_stacking ();
-  gtk_widget_show (win7);
+  gtk_window_present (GTK_WINDOW (win7));
 
   win8 = test_input_stacking ();
-  gtk_widget_show (win8);
+  gtk_window_present (GTK_WINDOW (win8));
 
   while (TRUE)
     g_main_context_iteration (NULL, TRUE);
